﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace i384
{
    public partial class Form1 : Form
    {
        private List<Vertex> vertices = new List<Vertex>();

        private List<Edge> edges = new List<Edge>();

        private int moveIndex = -1;

        private bool mouseMoved = false;

        Stopwatch selectionElapsedTime = new Stopwatch();

        private const long DOUBLE_CLICK_TIME = 500;

        private int selectedIndex = -1;

        public Form1()
        {
            InitializeComponent();
        }

        private void panel1_Paint(object sender, PaintEventArgs e)
        {
            try
            {
                Graphics graphics = e.Graphics;
                graphics.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias;

                for (int i = 0; i < vertices.Count; i++)
                {
                    Color color = i == selectedIndex ? Color.Gray : Color.Black;
                    vertices[i].Draw(graphics, i + 1, color);
                }

                foreach (Edge edge in edges)
                {
                    edge.Draw(graphics, vertices);
                }
            }
            catch (Exception)
            {

            }
        }

        int GetClickedVertex(Point position)
        {
            int[] clickedVertices = vertices
                .Select((vertex, index) => new Tuple<int, Vertex>(index, vertex))
                .Where(vertex => (vertex.Item2.Position + DrawingSettings.CircleSize / 2 - new Vector2(position)).Length < DrawingSettings.CircleSize.X / 2)
                .Select(indexedVertex => indexedVertex.Item1)
                .ToArray();

            return clickedVertices.Length > 0 ? clickedVertices.First() : -1;
        }

        private void panel1_MouseMove(object sender, MouseEventArgs e)
        {
            if (moveIndex > -1)
            {
                vertices[moveIndex].Position = new Vector2(e.Location) - DrawingSettings.CircleSize / 2;
            }

            mouseMoved = true;

            this.panel1.Refresh();
        }

        private void panel1_MouseUp(object sender, MouseEventArgs e)
        {
            int clickedVertex = GetClickedVertex(e.Location);

            if (clickedVertex < 0)
            {
                // add vertex
                vertices.Add(new Vertex(new Vector2(e.Location) - DrawingSettings.CircleSize / 2));

                selectedIndex = -1;
            }
            else if (selectedIndex > -1)
            {
                if (clickedVertex == selectedIndex)
                {
                    // remove vertex
                    if (selectionElapsedTime.ElapsedMilliseconds < DOUBLE_CLICK_TIME)
                    {
                        vertices.RemoveAt(selectedIndex);

                        // remove corresponding edges
                        edges = edges
                            .Where(edge => edge.StartingVertex != selectedIndex && edge.EndingVertex != selectedIndex)
                            .ToList();

                        // decrease vertex indices
                        for (int i = 0; i < edges.Count; i++)
                        {
                            if (edges[i].StartingVertex > selectedIndex)
                            {
                                edges[i].StartingVertex--;
                            }

                            if (edges[i].EndingVertex > selectedIndex)
                            {
                                edges[i].EndingVertex--;
                            }
                        }
                    }
                }
                else
                {
                    // find matching edges
                    Edge[] matches = edges
                        .Where(edge => (edge.StartingVertex == selectedIndex && edge.EndingVertex == clickedVertex) ||
                            (edge.StartingVertex == clickedVertex && edge.EndingVertex == selectedIndex))
                        .ToArray();

                    if (matches.Length > 0)
                    {
                        // remove edges
                        edges = edges.Except(matches).ToList();
                    }
                    else
                    {
                        // add edge
                        edges.Add(new Edge
                        {
                            Directed = e.Button == MouseButtons.Right,
                            StartingVertex = selectedIndex,
                            EndingVertex = clickedVertex
                        });
                    }
                }

                selectedIndex = -1;
            }
            else if (!mouseMoved)
            {
                selectedIndex = clickedVertex;
                selectionElapsedTime.Restart();
            }

            moveIndex = -1;

            this.panel1.Refresh();
        }

        private void panel1_MouseDown(object sender, MouseEventArgs e)
        {
            int clickedVertex = GetClickedVertex(e.Location);

            if (clickedVertex > -1)
            {
                moveIndex = clickedVertex;
            }

            mouseMoved = false;

            panel1.Refresh();
        }

        private void Form1_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == 'm' || e.KeyChar == 'M')
            {
                StringBuilder file = new StringBuilder();

                file.AppendFormat("{0} {1} {2} {3}", vertices.Count, edges.Count, this.Width, this.Height);
                file.AppendLine();
                file.AppendLine(string.Join(Environment.NewLine, vertices.Select(vertex => string.Format("{0} {1}", (int)vertex.Position.X, (int)vertex.Position.Y))));
                // two spaces if directed, one if not
                file.AppendLine(string.Join(Environment.NewLine, edges.Select(edge => string.Format("{0} {1}{2}", edge.StartingVertex, edge.Directed ? " " : "", edge.EndingVertex))));

                System.IO.File.WriteAllText("graf.txt", file.ToString());
            }

            if (e.KeyChar == 'b' || e.KeyChar == 'B')
            {
                if (!System.IO.File.Exists("graf.txt")) return;

                string[] lines = System.IO.File.ReadAllLines("graf.txt");
                int[] header = lines[0].Split(' ').Select(word => int.Parse(word)).ToArray();

                this.Size = new Size(header[2], header[3]);
                selectedIndex = -1;
                selectionElapsedTime.Reset();
                mouseMoved = false;
                moveIndex = -1;

                vertices = lines
                    .Skip(1)
                    .Take(header[0])
                    .Select(line => new Vertex(new Vector2(int.Parse(line.Split(' ')[0]), int.Parse(line.Split(' ')[1]))))
                    .ToList();

                edges.Clear();
                foreach (string line in lines.Skip(1 + header[0]).Take(header[1]))
                {
                    string[] numbers = line.Split(new char[] { ' ' }, StringSplitOptions.RemoveEmptyEntries);

                    Edge edge = new Edge
                    {
                        StartingVertex = int.Parse(numbers[0]),
                        EndingVertex = int.Parse(numbers[1]),
                        Directed = line.TrimStart().SkipWhile(ch => ch != ' ').TakeWhile(ch => ch ==  ' ').Count() == 2
                    };

                    edges.Add(edge);
                }
            }
        }
    }
}
